/* 	Ticket.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 26-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 29, 2008	dennis@brains2b.nl		Added javadoc and cleaned up		
*/
package org.brains2b.coffee;

import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.monitor.ProgressMonitor;
import org.brains2b.log.Logger;
import org.brains2b.task.DefaultTask;

/**
 * A Ticket is created by the TicketBroker as a result of an Event
 * that should be handled through the que. 
 * <p>The TicketHandler is supplied, so when the Ticket can be handled
 * from the queue it can just process the Event directly
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.1 [29-02-2008]
 * @see TicketBroker
 * @see TicketFarm
 * @see TicketQueue
 */
public class Ticket extends DefaultTask {
    
	/** type of result indicating that this Ticket 
	 * was removed before starting
	 */
	public final static int RESULT_REMOVE=4;
    
	/**
	 * Implementation of ProgressMonitor to supply
	 * progress information from TicketHandler regarding
	 * the running Task to this Ticket and TaskListener(s)
	 */
    private class Monitor implements ProgressMonitor {
        
        public void advance(int start, int value, int end) {
            if (getMax()!=end) {
            	setMin(start);
            	setMax(end);
            } 
            progress(value);
            
        }
    }
    
    private int m_sequence;
    private boolean m_wait;
    private TicketHandler m_ticketHandler;
    private CoffeeEvent m_event;
	private NNTPMessage m_message;
   
    
    /**
     * Constructor
     * @param server TicketHandler
     * @param event CoffeeEvent
     */
	public Ticket(TicketHandler server,CoffeeEvent event ) {
	    m_ticketHandler=server;
	    m_event=event;
    }
    
    public void run() {
        Monitor m=new Monitor();
        m_ticketHandler.addProgressMonitor(m);
        setFinished(false);
        try {
            m_message = m_ticketHandler.handleEvent(m_event);
        	setResult(RESULT_OK);
        } catch (Exception ex) {
            setResult(RESULT_ERROR);
            Logger.printException(ex);
        } catch (Throwable t) {
            setResult(RESULT_ERROR);
            t.printStackTrace();
        } finally {
            m_ticketHandler.removeProgressMonitor(m);
            setFinished(true);
        }
    }
    
    /**
     * the NNTPMessage which was is the result of this Ticket
     * @return NNTPMessage
     */
    public NNTPMessage getMessage() {
        return m_message;
    }
   
    /**
     * the CoffeeEvent which lead to creating this Ticket
     * @return CoffeeEvent
     */
    public CoffeeEvent getEvent() {
        return m_event;
    }
    
    /**
     * get the sequence of the Ticket
     * @return int, the sequence
     */
    public int getSequence() {
        return m_sequence;
    }

    /**
     * set the sequence number of this Ticket
     * @param sequence int, the sequence number as determined
     * by {@link TicketQueue}
     */
    void setSequence(int sequence) {
        m_sequence = sequence;
    }

    /**
     * get the TicketHandler which will handle this 
     * Ticket
     * @return {@link TicketHandler}
     */
    public TicketHandler getTicketHandler() {
        return m_ticketHandler;
    }
    
    /**
     * two tickets are considered equal if they have the 
     * same sequence number
     * @return boolean ture if Tickets are equal
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof Ticket)) return false;
        Ticket t= (Ticket) obj;
        if (m_sequence==t.getSequence()) {
            return true;
        } 
        return false;
    }

    /**
     * check if this Ticket is paused
     * @return boolean, true if paused
     */
	public boolean isWait() {
		return m_wait;
	}

	/**
	 * set this Ticket to pause, until released
	 * @param wait boolean, true to wait,false to release
	 */
	public void setWait(boolean wait) {
		m_wait = wait;
	}
	
	/**
	 * overridden to make public for {@link TicketQueue}
	 */
	public void setResult(int result) {
		super.setResult(result);
	}
}